@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/pool_connection_pool.c,v 1.4 2007/02/12 02:32:08 y-asaba Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool_connection_pool.c,v 1.6 2007/04/26 05:26:55 y-asaba Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -23,6 +23,7 @@
 #include "config.h"
 
 #include <sys/types.h>
+#include <sys/time.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <sys/un.h>
@@ -422,11 +423,18 @@ int connect_unix_domain_socket_by_port(int port, char *socket_dir)
 	snprintf(addr.sun_path, sizeof(addr.sun_path), "%s/.s.PGSQL.%d", socket_dir, port);
 	len = sizeof(struct sockaddr_un);
 
-	if (connect(fd, (struct sockaddr *)&addr, len) < 0)
+	for (;;)
 	{
-		pool_error("connect_unix_domain_socket_by_port: connect() failed: %s", strerror(errno));
-		close(fd);
-		return -1;
+		if (connect(fd, (struct sockaddr *)&addr, len) < 0)
+		{
+			if (errno == EINTR || errno == EAGAIN)
+				continue;
+
+			pool_error("connect_unix_domain_socket_by_port: connect() failed: %s", strerror(errno));
+			close(fd);
+			return -1;
+		}
+		break;
 	}
 
 	return fd;
@@ -474,11 +482,18 @@ int connect_inet_domain_socket_by_port(char *host, int port)
 			(char *) hp->h_addr,
 			hp->h_length);
 
-	if (connect(fd, (struct sockaddr *)&addr, len) < 0)
+	for (;;)
 	{
-		pool_error("connect_inet_domain_socket: connect() failed: %s",strerror(errno));
-		close(fd);
-		return -1;
+		if (connect(fd, (struct sockaddr *)&addr, len) < 0)
+		{
+			if (errno == EINTR || errno == EAGAIN)
+				continue;
+
+			pool_error("connect_inet_domain_socket: connect() failed: %s",strerror(errno));
+			close(fd);
+			return -1;
+		}
+		break;
 	}
 
 	return fd;
